<?php
// +----------------------------------------------------------------------
// | Leaps PHP Framework
// +----------------------------------------------------------------------
// | Copyright (c) 2010 http://www.newsteng.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: Silence <xutongle@gmail.com>
// +----------------------------------------------------------------------
// $Id$

/**
 +------------------------------------------------------------------------------
 * connect_qq.class.php Leaps QQ空间SDK
 +------------------------------------------------------------------------------
 * $Id$
 +------------------------------------------------------------------------------
 */
class connect_qq {

	public $appid,$appkey,$callback;

	function __construct() {
		self::set();
	}
	/**
	 * 初始化接口
	 *
	 */
	public static function set(){

	}
	/**
	 * 所有Get请求都可以使用这个方法
	 *
	 * @param $url
	 * @param $appid
	 * @param $appkey
	 * @param $access_token
	 * @param $access_token_secret
	 * @param $openid
	 *
	 * @return true or false
	 */
	public static function get($url, $appid, $appkey, $access_token, $access_token_secret, $openid) {
    	$sigstr = "GET"."&".rawurlencode("$url")."&";
    	//必要参数, 不要随便更改!!
    	$params = $_GET;
    	$params["oauth_version"]          = "1.0";
    	$params["oauth_signature_method"] = "HMAC-SHA1";
    	$params["oauth_timestamp"]        = time();
    	$params["oauth_nonce"]            = mt_rand();
    	$params["oauth_consumer_key"]     = $appid;
    	$params["oauth_token"]            = $access_token;
    	$params["openid"]                 = $openid;
    	unset($params["oauth_signature"]);

    	//参数按照字母升序做序列化
    	$normalized_str = self::get_normalized_string($params);
    	$sigstr        .= rawurlencode($normalized_str);

    	//签名,确保php版本支持hash_hmac函数
    	$key = $appkey."&".$access_token_secret;
    	$signature = self::get_signature($sigstr, $key);
    	$url      .= "?".$normalized_str."&"."oauth_signature=".rawurlencode($signature);
    	return file_get_contents($url);
	}

	/**
	 * 所有multi-part post 请求都可以使用这个方法
	 *
	 * @param $url
	 * @param $appid
	 * @param $appkey
	 * @param $access_token
	 * @param $access_token_secret
	 * @param $openid
	 *
	 */
	function multi_post($url, $appid, $appkey, $access_token, $access_token_secret, $openid) {
    	//构造签名串.源串:方法[GET|POST]&uri&参数按照字母升序排列
    	$sigstr = "POST"."&"."$url"."&";

    	//必要参数,不要随便更改!!
    	$params = $_POST;
    	$params["oauth_version"]          = "1.0";
    	$params["oauth_signature_method"] = "HMAC-SHA1";
    	$params["oauth_timestamp"]        = time();
    	$params["oauth_nonce"]            = mt_rand();
    	$params["oauth_consumer_key"]     = $appid;
    	$params["oauth_token"]            = $access_token;
    	$params["openid"]                 = $openid;
    	unset($params["oauth_signature"]);


    	//获取上传图片信息
    	foreach ($_FILES as $filename => $filevalue) {
        	if ($filevalue["error"] != UPLOAD_ERR_OK) {
            //echo "upload file error $filevalue['error']\n";
            //exit;
        	}
        	$params[$filename] = file_get_contents($filevalue["tmp_name"]);
    	}

    	//对参数按照字母升序做序列化
    	$sigstr .= self::get_normalized_string($params);

    	//签名,需要确保php版本支持hash_hmac函数
    	$key = $appkey."&".$access_token_secret;
    	$signature = self::get_signature($sigstr, $key);
    	$params["oauth_signature"] = $signature;

    	//处理上传图片
    	foreach ($_FILES as $filename => $filevalue) {
        	$tmpfile = dirname($filevalue["tmp_name"])."/".$filevalue["name"];
        	move_uploaded_file($filevalue["tmp_name"], $tmpfile);
        	$params[$filename] = "@$tmpfile";
    	}
    	$ch = curl_init();
    	curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    	curl_setopt($ch, CURLOPT_POST, TRUE);
    	curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
    	curl_setopt($ch, CURLOPT_URL, $url);
    	$ret = curl_exec($ch);
    	curl_close($ch);
    	//删除上传临时文件
    	unlink($tmpfile);
    	return $ret;
	}

	/**
	 * 检查openid是否合法
	 *
	 * @param int $openid 与用户QQ号码一一对应
	 * @param int $timestamp 时间戳
	 * @param string $sig 签名值
	 * @return true or false
	 */
	public static function is_valid_openid($openid, $timestamp, $sig) {
    	$key = $_SESSION["appkey"];
    	$str = $openid.$timestamp;
    	$signature = self::get_signature($str, $key);
    	return $sig == $signature;
	}

	/**
	 * 对字符串进行URL编码，遵循rfc1738 urlencode
	 *
	 * @param string $params
	 * @return URL编码后的字符串
	 */
	public static function get_urlencode_string($params) {
    	ksort($params);
    	$normalized = array();
    	foreach($params as $key => $val){
        	$normalized[] = $key."=".rawurlencode($val);
    	}
    	return implode("&", $normalized);
	}

	/**
	 * 使用HMAC-SHA1算法生成oauth_signature签名值
	 *
	 * @param string $str 密钥
	 * @param string $key 源串
	 * @return 签名值
	 */
	public static function get_signature($str, $key) {
    	$signature = "";
    	if (function_exists('hash_hmac')) {
        	$signature = base64_encode(hash_hmac("sha1", $str, $key, true));
    	} else {
        	$blocksize	= 64;
        	$hashfunc	= 'sha1';
        	if (strlen($key) > $blocksize) {
            	$key = pack('H*', $hashfunc($key));
        	}
        	$key	= str_pad($key,$blocksize,chr(0x00));
        	$ipad	= str_repeat(chr(0x36),$blocksize);
        	$opad	= str_repeat(chr(0x5c),$blocksize);
        	$hmac 	= pack('H*',$hashfunc(($key^$opad).pack('H*',$hashfunc(($key^$ipad).$str))));
        	$signature = base64_encode($hmac);
    	}
    	return $signature;
	}

	/**
	 * 对参数进行字典升序排序
	 *
	 * @param string $params 参数列表
	 * @return  排序后用&链接的key-value对（key1=value1&key2=value2...)
	 */
	public static function get_normalized_string($params){
		ksort($params);
    	$normalized = array();
    	foreach($params as $key => $val) {
        	$normalized[] = $key."=".$val;
    	}
    	return implode("&", $normalized);
	}

}


/* End of file connect_qq.class.php */